/*
 * Copyright (c) 2005-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: XBaya.java,v 1.30 2007/07/25 14:59:36 sshirasu Exp $
 */

package edu.indiana.extreme.xbaya;

import java.net.URI;
import java.net.URISyntaxException;

import xsul5.MLogger;
import edu.indiana.extreme.xbaya.gui.ErrorMessages;

/**
 * @author Satoshi Shirasuna
 */
public class XBaya {

    private static MLogger logger = MLogger.getLogger();

    private XBayaConfiguration config;

    private XBayaEngine engine;

    /**
     * Constructs an XBayaEngine.
     * 
     * @param args
     */
    public XBaya(String[] args) {
        parseArguments(args);
        try {
            this.engine = new XBayaEngine(this.config);
        } catch (RuntimeException e) {
            logger.caught(e);
            try {
                this.engine.getErrorWindow().error(
                        ErrorMessages.UNEXPECTED_ERROR, e);
            } catch (Throwable t) {
                // Cannot do anything
            }
        } catch (Error e) {
            logger.caught(e);
            try {
                this.engine.getErrorWindow().error(
                        ErrorMessages.UNEXPECTED_ERROR, e);
            } catch (Throwable t) {
                // Cannot do anything
            }
        }
    }

    /**
     * Returns the XBayaEngine.
     * 
     * @return The XBayaEngine
     */
    public XBayaEngine getEngine() {
        return this.engine;
    }

    private void printUsage() {
        System.err.println("Usage: java " + XBaya.class.getName() + " [-help]"
                + "[-config file]" + " [-title title]"
                + " [-workflow workflow]" + " [-enableLocalRegistry]"
                + " [-localRegistry dir]" + " [-gpelEngineURL url]"
                + " [-gpelTemplateID templateID]"
                + " [-gpelInstanceID instanceID]"
                + " [-resourceCatalogURL url]" + " [-xRegistryURL url]"
                + " [-gfacRegistryURL url]" + " [-gfacURL url]"
                + " [-dscURL url" + " [-myLeadAgentURL url]"
                + " [-myLeadUser userDN]" + " [-myLeadProject projectID]"
                + " [-myLeadWorkflow workflow]" + " [-myLeadSampleUser userDN]"
                + " [-myLeadSampleProject projectID]"
                + " [-startMonitor {true,false}]" + " [-brokerURL url]"
                + " [-topic topic]" + " [-pullMode {true,false}]"
                + " [-myProxyServer host]" + " [-karmaURL url]"
                + " [-karmaWorkflowInstanceID]" + " [-myProxyPort port]"
                + " [-myProxyUsername username]" + " [-myProxyLifetime sec]"
                + " [-loadMyProxy {true,false}]" + " [-messageBoxURL url]"
                + " [-width width]" + " [-height height]");
    }

    private void parseArguments(String[] args) {
        try {
            this.config = new XBayaConfiguration();

            int index = 0;
            while (index < args.length) {
                String arg = args[index];
                String possibleValue = "";
                if ((index + 1) < args.length) {
                    possibleValue = args[index + 1];
                }
                logger.info("arg: " + arg + " " + possibleValue);
                if ("-help".equalsIgnoreCase(arg)) {
                    printUsage();
                    System.exit(0);
                } else if ("-config".equalsIgnoreCase(arg)) {
                    index++;
                    String configPath = args[index];
                    try {
                        this.config.loadConfiguration(configPath);
                    } catch (RuntimeException e) {
                        String message = "Error while reading config file, "
                                + configPath;
                        logger.warning(message, e);
                        this.config.addError(new XBayaException(message, e));
                    }
                } else if ("-title".equalsIgnoreCase(arg)) {
                    index++;
                    this.config.setTitle(args[index]);
                } else if ("-workflow".equalsIgnoreCase(arg)) {
                    index++;
                    this.config.setWorkflow(args[index]);
                } else if ("-enableLocalRegistry".equalsIgnoreCase(arg)) {
                    this.config.enableLocalRegistry();
                } else if ("-localRegistry".equalsIgnoreCase(arg)) {
                    index++;
                    String path = args[index];
                    this.config.addLocalRegistry(path);
                } else if ("-gpelEngineURL".equalsIgnoreCase(arg)) {
                    index++;
                    String gpelEngineURL = args[index];
                    try {
                        this.config.setGPELEngineURL(parseURL(gpelEngineURL));
                    } catch (URISyntaxException e) {
                        String message = "The GPEL Engine URL is in wrong format: "
                                + gpelEngineURL;
                        logger.warning(message, e);
                        this.config.addError(new XBayaException(message, e));
                    }
                } else if ("-gpelUser".equalsIgnoreCase(arg)) {
                    index++;
                    logger.warning("gpelUser is obsolete");
                } else if ("-gpelPassword".equalsIgnoreCase(arg)) {
                    index++;
                    logger.warning("gpelUser is obsolete");
                } else if ("-gpelTemplateID".equalsIgnoreCase(args[index])) {
                    index++;
                    String templateID = args[index];
                    try {
                        this.config.setGPELTemplateID(parseURL(templateID));
                    } catch (URISyntaxException e) {
                        String message = "The GPEL Template ID is in wrong format: "
                                + templateID;
                        logger.warning(message, e);
                        this.config.addError(new XBayaException(message, e));
                    }
                } else if ("-gpelInstanceID".equalsIgnoreCase(args[index])) {
                    index++;
                    String instanceID = args[index];
                    try {
                        this.config.setGPELInstanceID(parseURL(instanceID));
                    } catch (URISyntaxException e) {
                        String message = "The GPEL Instance ID is in wrong format: "
                                + instanceID;
                        logger.warning(message, e);
                        this.config.addError(new XBayaException(message, e));
                    }
                } else if ("-resourceCatalogURL".equalsIgnoreCase(arg)) {
                    index++;
                    String url = args[index];
                    try {
                        this.config.setResourceCatalogURL(parseURL(url));
                    } catch (URISyntaxException e) {
                        String message = "The Resource Catalog URL is in wrong format: "
                                + url;
                        logger.warning(message, e);
                        this.config.addError(new XBayaException(message, e));
                    }
                } else if ("-xRegistryURL".equalsIgnoreCase(arg)) {
                    index++;
                    String url = args[index];
                    try {
                        this.config.setXRegistryURL(parseURL(url));
                    } catch (URISyntaxException e) {
                        String message = "The XRegistry URL is in wrong format: "
                                + url;
                        logger.warning(message, e);
                        this.config.addError(new XBayaException(message, e));
                    }
                } else if ("-gfacRegistryURL".equalsIgnoreCase(arg)) {
                    index++;
                    String url = args[index];
                    try {
                        this.config.setGFacRegistryURL(parseURL(url));
                    } catch (URISyntaxException e) {
                        String message = "The GFac Registry URL is in wrong format: "
                                + url;
                        logger.warning(message, e);
                        this.config.addError(new XBayaException(message, e));
                    }
                } else if ("-gfacURL".equalsIgnoreCase(arg)) {
                    index++;
                    String url = args[index];
                    try {
                        this.config.setGFacURL(parseURL(url));
                    } catch (URISyntaxException e) {
                        String message = "The GFac URL is in wrong format: "
                                + url;
                        logger.warning(message, e);
                        this.config.addError(new XBayaException(message, e));
                    }
                } else if ("-dscURL".equalsIgnoreCase(arg)) {
                    index++;
                    String url = args[index];
                    try {
                        this.config.setDSCURL(parseURL(url));
                    } catch (URISyntaxException e) {
                        String message = "The DSC URL is in wrong format: "
                                + url;
                        logger.warning(message, e);
                        this.config.addError(new XBayaException(message, e));
                    }
                } else if ("-myLeadAgentURL".equals(arg)
                        || "-leadURL".equalsIgnoreCase(arg)) {
                    index++;
                    String url = args[index];
                    try {
                        this.config.setMyLeadAgentURL(parseURL(url));
                    } catch (URISyntaxException e) {
                        String message = "The myLead agent URL is in wrong format: "
                                + url;
                        logger.warning(message, e);
                        this.config.addError(new XBayaException(message, e));
                    }
                } else if ("-myLeadUser".equalsIgnoreCase(arg)
                        || "-leadUID".equalsIgnoreCase(arg)) {
                    index++;
                    this.config.setMyLeadUser(args[index]);
                } else if ("-myLeadProject".equals(arg)
                        || "-leadProj".equalsIgnoreCase(arg)) {
                    index++;
                    this.config.setMyLeadProject(args[index]);
                } else if ("-myLeadWorkflow".equalsIgnoreCase(arg)
                        || "-leadWorkflow".equalsIgnoreCase(arg)) {
                    index++;
                    this.config.setMyLeadWorkflow(args[index]);
                } else if ("-myLeadSampleUser".equalsIgnoreCase(arg)
                        || "-leadSampleUID".equalsIgnoreCase(arg)) {
                    index++;
                    this.config.setMyLeadSampleUser(args[index]);
                } else if ("-myLeadSampleProject".equalsIgnoreCase(arg)
                        || "-leadSampleProj".equalsIgnoreCase(arg)) {
                    index++;
                    this.config.setMyLeadSampleProject(args[index]);
                } else if ("-startMonitor".equalsIgnoreCase(arg)) {
                    this.config.setStartMonitor(true);
                } else if ("-brokerURL".equalsIgnoreCase(arg)) {
                    index++;
                    String brokerURL = args[index];
                    try {
                        this.config.setBrokerURL(parseURL(brokerURL));
                    } catch (URISyntaxException e) {
                        String message = "The broker URL is in wrong format: "
                                + brokerURL;
                        logger.warning(message, e);
                        this.config.addError(new XBayaException(message, e));
                    }
                } else if ("-topic".equalsIgnoreCase(arg)) {
                    index++;
                    this.config.setTopic(args[index]);
                } else if ("-pullMode".equalsIgnoreCase(arg)) {
                    if (index < args.length - 1) {
                        String nextArg = args[index + 1];
                        if (nextArg.startsWith("-")) {
                            this.config.setPullMode(true);
                        } else if ("true".equalsIgnoreCase(nextArg)) {
                            index++;
                            this.config.setPullMode(true);
                        } else if ("false".equalsIgnoreCase(nextArg)) {
                            index++;
                            this.config.setPullMode(false);
                        } else {
                            String message = "-pullMode has to be either true or false, not "
                                    + nextArg;
                            logger.warning(message);
                            this.config.addError(new XBayaException(message));
                        }
                    } else {
                        // This is the last arg
                        this.config.setPullMode(true);
                    }
                } else if ("-messageBoxURL".equalsIgnoreCase(arg)
                        || "-msgBoxURL".equalsIgnoreCase(arg)) {
                    index++;
                    String messageBoxURL = args[index];
                    try {
                        this.config.setMessageBoxURL(parseURL(messageBoxURL));
                    } catch (URISyntaxException e) {
                        String message = "The message box URL is in wrong format: "
                                + messageBoxURL;
                        logger.warning(message, e);
                        this.config.addError(new XBayaException(message, e));
                    }
                } else if ("-karmaURL".equalsIgnoreCase(arg)) {
                    index++;
                    String karmaURL = args[index];
                    try {
                        this.config.setKarmaURL(parseURL(karmaURL));
                    } catch (URISyntaxException e) {
                        String message = "The Karma URL is in wrong format: "
                                + karmaURL;
                        logger.warning(message, e);
                        this.config.addError(new XBayaException(message, e));
                    }
                } else if ("-karmaWorkflowInstanceID"
                        .equalsIgnoreCase(args[index])) {
                    index++;
                    String instanceID = args[index];
                    try {
                        this.config
                                .setKermaWorkflowInstanceID(parseURL(instanceID));
                    } catch (URISyntaxException e) {
                        String message = "The Karma workflow instance ID is in wrong format: "
                                + instanceID;
                        logger.warning(message, e);
                        this.config.addError(new XBayaException(message, e));
                    }
                } else if ("-myProxyServer".equalsIgnoreCase(arg)) {
                    index++;
                    String server = args[index];
                    if ("null".equalsIgnoreCase(server)) {
                        // This is a workaround that JNLP doesn't take empty
                        // string as an argument.
                        server = null;
                    }
                    this.config.setMyProxyServer(server);
                } else if ("-myProxyPort".equalsIgnoreCase(arg)) {
                    index++;
                    String port = args[index];
                    try {
                        this.config.setMyProxyPort(Integer.parseInt(port));
                    } catch (NumberFormatException e) {
                        String message = "The myProxyPort must be an integer: "
                                + port;
                        logger.warning(message, e);
                        this.config.addError(new XBayaException(message, e));
                    }
                } else if ("-myProxyUsername".equalsIgnoreCase(arg)) {
                    index++;
                    this.config.setMyProxyUsername(args[index]);
                } else if ("-myProxyPassphrase".equalsIgnoreCase(arg)) {
                    // This is only for debugging.
                    index++;
                    this.config.setMyProxyPassphrase(args[index]);
                } else if ("-myProxyLifetime".equalsIgnoreCase(arg)) {
                    index++;
                    String lifetime = args[index];
                    try {
                        this.config.setMyProxyLifetime(Integer
                                .parseInt(lifetime));
                    } catch (NumberFormatException e) {
                        String message = "The myProxyLifetime must be an integer: "
                                + lifetime;
                        logger.warning(message, e);
                        this.config.addError(new XBayaException(message, e));
                    }
                } else if ("-loadMyProxy".equalsIgnoreCase(arg)) {
                    index++;
                    String load = args[index];
                    if ("true".equalsIgnoreCase(load)) {
                        this.config.setLoadMyProxy(true);
                    } else if ("false".equalsIgnoreCase(load)) {
                        this.config.setLoadMyProxy(false);
                    } else {
                        String message = "-loadMyProxy has to be either true or false, not "
                                + load;
                        logger.warning(message);
                        this.config.addError(new XBayaException(message));
                    }
                } else if ("-width".equalsIgnoreCase(arg)) {
                    index++;
                    String width = args[index];
                    try {
                        this.config.setWidth(Integer.parseInt(width));
                    } catch (NumberFormatException e) {
                        String message = "The width must be an integer: "
                                + width;
                        logger.warning(message, e);
                        this.config.addError(new XBayaException(message, e));
                    }
                } else if ("-height".equalsIgnoreCase(arg)) {
                    index++;
                    String height = args[index];
                    try {
                        this.config.setHeight(Integer.parseInt(height));
                    } catch (NumberFormatException e) {
                        String message = "The height must be an integer: "
                                + height;
                        logger.warning(message, e);
                        this.config.addError(new XBayaException(message, e));
                    }
                } else {
                    String message = "Unknown option: " + arg;
                    logger.severe(message);
                    this.config.addError(new XBayaException(message));
                }
                index++;
            }
        } catch (ArrayIndexOutOfBoundsException e) {
            String message = "Argument is missing after "
                    + args[args.length - 1];
            logger.severe(message, e);
            this.config.addError(new XBayaException(message));
        } catch (Throwable e) {
            logger.caught(e);
            String message = "Unknown error while parsing the arguments";
            this.config.addError(new XBayaException(message, e));
        }
    }

    private URI parseURL(String urlString) throws URISyntaxException {
        if (urlString.trim().length() == 0) {
            // This makes it possible to not use some of our default services.
            return null;
        } else if ("null".equalsIgnoreCase(urlString)) {
            // This is a workaround that JNLP doesn't take empty string as an
            // argument.
            return null;
        } else {
            return new URI(urlString).parseServerAuthority();
        }
    }

    /**
     * @param args
     */
    public static void main(String[] args) {
        new XBaya(args);
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2005-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */
